import argparse
import os

import numpy as np
import pydicom

DCM_TAG_PIXEL_SCALE_FACTOR = 0x00331038

header = """
!INTERFILE  :=
!imaging modality := nucmed
!version of keys := STIR4.0
name of data file := {DATA_FILE}
!GENERAL DATA :=
!GENERAL IMAGE DATA :=
!type of data := Tomographic
imagedata byte order := LITTLEENDIAN
isotope name := ^99m^Technetium
!SPECT STUDY (General) :=
process status := Reconstructed
!number format := float
!number of bytes per pixel := 4
number of dimensions := 3
matrix axis label [1] := x
!matrix size [1] := {ROWS}
scaling factor (mm/pixel) [1] := {SPACING_X}
matrix axis label [2] := y
!matrix size [2] := {COLUMNS}
scaling factor (mm/pixel) [2] := {SPACING_Y}
matrix axis label [3] := z
!matrix size [3] := {SLICES}
scaling factor (mm/pixel) [3] := {SPACING_Z}
first pixel offset (mm) [1] := {OFFSET_X}
first pixel offset (mm) [2] := {OFFSET_Y}
first pixel offset (mm) [3] := 0
number of time frames := 1
!END OF INTERFILE :=
"""

parser = argparse.ArgumentParser(description="convert a DICOM image to the INTERFILE format")
parser.add_argument("--dcm", type=str, required=True, help="the DICOM image to convert")
parser.add_argument("--out", type=str, help="the filename for the resulting INTERFILE. If not specified it defaults to the input name with different extensions.")
args = parser.parse_args()

if args.out is None:
    args.out = os.path.splitext(args.dcm)[0]
print(args)

dcm = pydicom.dcmread(args.dcm)

datafile = f"{args.out}.v"
data = dcm.pixel_array / dcm[DCM_TAG_PIXEL_SCALE_FACTOR].value
data = data.astype(np.float32)
with open(datafile, mode="wb") as f:
    f.write(data.tobytes())

header = header.replace("{DATA_FILE}", os.path.basename(datafile))
header = header.replace("{ROWS}", str(dcm.Rows))
header = header.replace("{SPACING_X}", f"{dcm.PixelSpacing[0]:.4f}")
header = header.replace("{COLUMNS}", str(dcm.Columns))
header = header.replace("{SPACING_Y}", f"{dcm.PixelSpacing[1]:.4f}")
header = header.replace("{SLICES}", str(dcm.NumberOfSlices))
header = header.replace("{SPACING_Z}", f"{dcm.SliceThickness:.4f}")
header = header.replace("{OFFSET_X}", str(-0.5 * dcm.Rows * dcm.PixelSpacing[0]))
header = header.replace("{OFFSET_Y}", str(-0.5 * dcm.Columns * dcm.PixelSpacing[1]))
header = header.strip()

headerfile = f"{args.out}.hv"
with open(headerfile, mode="w") as f:
    f.write(header)
