import argparse

import cv2 as cv
import numpy as np
import stir
import stirextra

from vis import to_gray

stir.Verbosity_set(-1)

# projdata = stir.ProjData.read_from_file("input.hs")
projdata = stir.ProjData.read_from_file("symbia/proj_ref.hs")
recon = stir.FloatVoxelsOnCartesianGrid.read_from_file("./symbia/recon_v2.hv")

projmatrix = stir.ProjMatrixByBinUsingRayTracing()
projmatrix.set_up(projdata.get_proj_data_info(), recon)

forwardprojector = stir.ForwardProjectorByBinUsingProjMatrixByBin(projmatrix)
backprojector = stir.BackProjectorByBinUsingProjMatrixByBin(projmatrix)

#%% create projection data for output of forward projection
# We'll just create the data in memory here
projdataout = stir.ProjDataInMemory(projdata.get_exam_info(),
                                    projdata.get_proj_data_info())
# Note: we could write to file, but it is right now a bit complicated to open a

forwardprojector.set_up(projdataout.get_proj_data_info(), recon)
forwardprojector.forward_project(projdataout, recon)

_recon = stirextra.to_numpy(recon)

out_bef = stirextra.to_numpy(projdataout.get_segment_by_sinogram(0))
proj = np.transpose(out_bef.copy(), (1, 0, 2))
i = 0
s = np.full((512, 10), 239, np.uint8)
s_full = np.full((512, 512), 239, np.uint8)
while True:
    _img = to_gray(proj[i], proj.min(), proj.max())
    _img = cv.putText(_img, str(i), (0, 30), cv.FONT_HERSHEY_SIMPLEX, 1, 255, 3)
    _img2 = to_gray(_recon[i % _recon.shape[0]], _recon.min(), _recon.max())

    i = (i + 1) % proj.shape[0]

    cv.imshow("Proj", np.hstack((_img, s, _img2)))
    key = cv.waitKey(100)
    if key == ord("q"):
        break
        

recon.fill(0)
backprojector.set_up(projdataout.get_proj_data_info(), recon)
backprojector.back_project(recon, projdataout)

recon = stirextra.to_numpy(recon)
i = 0
while True:
    _img = to_gray(proj[i], proj.min(), proj.max())
    _img = cv.putText(_img, str(i), (0, 30), cv.FONT_HERSHEY_SIMPLEX, 1, 255, 3)
    _img2 = to_gray(_recon[i % _recon.shape[0]], _recon.min(), _recon.max())
    _img3 = to_gray(recon[i % recon.shape[0]], recon.min(), recon.max())

    i = (i + 1) % proj.shape[0]

    row1 = np.hstack((_img, s, _img2))
    row2 = np.hstack((s_full, s, _img3))

    cv.imshow("Proj", np.vstack((row1, np.full((10, row1.shape[1]), 239, np.uint8), row2)))
    key = cv.waitKey(100)
    if key == ord("q"):
        break
#%% display
# This shows a beautiful pattern, a well-known feature of a ray-tracing matrix
# target_array = stirextra.to_numpy(target)
# pylab.figure()
# pylab.subplot(1, 2, 1)
# pylab.imshow(target_array[10, :, :])
# pylab.title('Back-projection')
# pylab.subplot(1, 2, 2)
# pylab.plot(target_array[10, 80, :])
# pylab.show(block=False)
#%% Let's use more LORs per sinogram bin (which will be a bit slower of course)
# projmatrix.set_num_tangential_LORs(10)
# Need to call set_up again
# projmatrix.set_up(projdata.get_proj_data_info(), target)
#%% You could re-run the forward projection, but we'll skip that for now
# forwardprojector.forward_project(projdataout, target);
#%% Run another backprojection and display
# target.fill(0)
# backprojector.back_project(target, projdataout)
# new_target_array = stirextra.to_numpy(target)

